%Benchmark case for gravity driven poiseuille flow to compare numerical and analytical
%solution. 

% Bounceback on walls

%INITIALIZATION
clear
NY=input('Number of cells in crossflow direction (NY):');
NX=NY;
h=NY-1;
u_lattice=1/h; %Or: u_L~delta y. delta y=1/h where h=(NY-1). This gives omega=const.
delta_y=1/h;
umax=1; %Macroscopic velocity
%uavg=(2/3)*umax;
Re=10;
nu=u_lattice*h/Re; 
tau=3*nu+0.5;
omega=1/tau;
% omega=1.0; %Relaxation frequency
% tau=1/omega; %Relaxation time
% nu=(1/3)*(tau-0.5); %Kinematic viscosity
% Re=4.4; %Reynolds number
% umax=0.1;
% uavg=2/3*umax;
% NY=round(Re*nu/uavg); %Lattice units in y-direction. Re is based on the average velocity. Min number of points
% NX=NY;
i=1:NX; %Computational domain:x-count
j=1:NY; %Computational domain:y-count
J=2:NY-1;%Wet nodes i y-direction
jx=zeros(NY,NX); %x-momentum
jy=zeros(NY,NX); %y-momentum
rho=ones(NY,NX); %Density
f=zeros(NY,NX); %Particle distribution function. f=f_{in}
feq=zeros(NY,NX,9);
fout=zeros(NY,NX);
g=8*u_lattice*nu/h^2; %Gravity. 
G=g*rho; %Forcing term
Gy=zeros(NY,NX);
Z=zeros(NY,NX,9); %Force term in collision. Eqn.(2.74) in Latt
%Analytical solution
C1=(4*umax/h^3)*(NY^2-1);
C2=4*umax/h^2 - C1;
uana=-(4*umax/h^2)*j.^2 + C1*j + C2;
%Other input variables
aa=1-0.5*omega;
bb=1-0.5*omega;

% maxitr=input('Number of iterations: ');

%D2Q9 lattice constants
t0=4/9; t1=1/9; t2=1/36;
t=[t1,t1,t1,t1,t2,t2,t2,t2,t0]; %Lattice weights
f0=1; f1=3; f2=4.5; f3=1.5;
cx=[1,0,-1,0,1,-1,-1,1,0];
cy=[0,1,0,-1,1,1,-1,-1,0];


%        C6  C2  C5       ^ y
%          \ | /          |
%         C3-C0-C1        | 
%          / | \          | 
%        C7  C4  C8         -----> x


ux=jx./rho; %Compute macroscopic x-velocity
uy=jy./rho; %Compute macroscopic y-velocity

uxsq=ux.*ux;
uysq=uy.*uy;

usq=uxsq + uysq;

uxuy5=ux + uy;
uxuy6=-ux + uy;
uxuy7=-ux - uy;
uxuy8=ux - uy;

% Initialize the particle distribution function to equilibrium: f(t=0)=f^{eq}
f(j,i,1)=t(1)*rho(j,i).*(f0 + f1.*ux(j,i)  + f2.*uxsq(j,i) -          f3*usq(j,i));
f(j,i,2)=t(2)*rho(j,i).*(f0 + f1.*uy(j,i)  + f2.*uysq(j,i) -          f3*usq(j,i));
f(j,i,3)=t(3)*rho(j,i).*(f0 - f1.*ux(j,i)  + f2.*uxsq(j,i) -          f3*usq(j,i));
f(j,i,4)=t(4)*rho(j,i).*(f0 - f1.*uy(j,i)  + f2.*uysq(j,i) -          f3*usq(j,i));
f(j,i,5)=t(5)*rho(j,i).*(f0 + f1.*uxuy5(j,i) + f2.*uxuy5(j,i).*uxuy5(j,i)-    f3*usq(j,i));
f(j,i,6)=t(6)*rho(j,i).*(f0 + f1.*uxuy6(j,i) + f2.*uxuy6(j,i).*uxuy6(j,i)-    f3*usq(j,i));
f(j,i,7)=t(7)*rho(j,i).*(f0 + f1.*uxuy7(j,i) + f2.*uxuy7(j,i).*uxuy7(j,i)-    f3*usq(j,i));
f(j,i,8)=t(8)*rho(j,i).*(f0 + f1.*uxuy8(j,i) + f2.*uxuy8(j,i).*uxuy8(j,i)-    f3*usq(j,i));
f(j,i,9)=t(9)*rho(j,i).*(f0 -                                       f3*usq(j,i));

fout=f;

difference=1;
n=1;
convergence_criterion=1e-12; %eps
%COLLISION PHASE
% err=ones(n,1);
maxitr=100000; %Maximum number of iterations
% for n=1:maxitr
err=1;

tic;
while difference(n)>convergence_criterion && n<maxitr
 
 %Update the values of rho, jx and jy
rho(j,i)=f(j,i,1)+f(j,i,2)+f(j,i,3)+f(j,i,4)+f(j,i,5)+f(j,i,6)+f(j,i,7)+f(j,i,8)+f(j,i,9);
% rho(j,i)=ones(NY,NX);
jx(j,i)=cx(1)*f(j,i,1);
jy(j,i)=cy(1)*f(j,i,1);
for a=2:8
jx(j,i)=jx(j,i) + cx(a)*f(j,i,a);
jy(j,i)=jy(j,i) + cy(a)*f(j,i,a);
end

G=rho.*g;

ux=jx./rho;
uy=jy./rho;
% uy=zeros(NY,NX);

% ux=ux+0.5*g*ones(NY,NX); %Eqn. (2.30) in Latt
ux=ux+0.5*G;

ueqx=ux; %The velocities used in f^{eq} depend on the equilibrium velocities.
ueqy=uy;

uxsq=ueqx.*ueqx;
uysq=ueqy.*ueqy;

usq=uxsq + uysq; %vector

u=usq.^0.5; %vector, to be used in Z,(2.74)

uxuy5=ux + uy;
uxuy6=-ux + uy;
uxuy7=-ux - uy;
uxuy8=ux - uy;
uxuy9=0.*ux+0.*uy;

%Compute the equilibrium function f^{eq}
feq(J,i,1)=t(1)*rho(J,i).*(f0 + f1.*ueqx(J,i)  + f2.*uxsq(J,i) -                 f3.*usq(J,i));
feq(J,i,2)=t(2)*rho(J,i).*(f0 + f1.*ueqy(J,i)  + f2.*uysq(J,i) -                 f3.*usq(J,i));
feq(J,i,3)=t(3)*rho(J,i).*(f0 - f1.*ueqx(J,i)  + f2.*uxsq(J,i) -                 f3.*usq(J,i));
feq(J,i,4)=t(4)*rho(J,i).*(f0 - f1.*ueqy(J,i)  + f2.*uysq(J,i) -                 f3.*usq(J,i));
feq(J,i,5)=t(5)*rho(J,i).*(f0 + f1.*uxuy5(J,i) + f2.*uxuy5(J,i).*uxuy5(J,i)-     f3.*usq(J,i));
feq(J,i,6)=t(6)*rho(J,i).*(f0 + f1.*uxuy6(J,i) + f2.*uxuy6(J,i).*uxuy6(J,i)-     f3.*usq(J,i));
feq(J,i,7)=t(7)*rho(J,i).*(f0 + f1.*uxuy7(J,i) + f2.*uxuy7(J,i).*uxuy7(J,i)-     f3.*usq(J,i));
feq(J,i,8)=t(8)*rho(J,i).*(f0 + f1.*uxuy8(J,i) + f2.*uxuy8(J,i).*uxuy8(J,i)-     f3.*usq(J,i));
feq(J,i,9)=t(9)*rho(J,i).*(f0 -                                                  f3.*usq(J,i));

% Q_inner_S_l(a)=trace(Qxy(a)'*Sxy_l);
%Z(j,i,a) is eqn.(2.74) in Latt
for k=1:9
    p(:,:,k)=(cx(k)*ones(NY,NX)-ux).*G + (cy(k)*ones(NY,NX)-uy).*Gy;
    q(:,:,k)=(cx(k)*ux*cx(k)).*G + (cy(k)*uy*cx(k)).*G + (cx(k)*ux*cy(k)).*Gy + (cy(k)*uy*cy(k)).*Gy;
    Z(:,:,k)=aa*t(k)*3*p(:,:,k) + aa*t(k)*9*q(:,:,k);
end


% Z(j,i,1)=aa*t(1)*(3*(cx(1)*ones(NY,NX)-u(j,i))                          + 9*(ux(j,i))).*G(j,i);
% Z(j,i,2)=aa*t(2)*(3*(cy(2)*ones(NY,NX)-u(j,i))                          + 9*(uy(j,i))).*G(j,i);
% Z(j,i,3)=aa*t(3)*(3*(cx(3)*ones(NY,NX)-u(j,i))                          + 9*(-ux(j,i))).*G(j,i);
% Z(j,i,4)=aa*t(4)*(3*(cy(4)*ones(NY,NX)-u(j,i))                          + 9*(-uy(j,i))).*G(j,i);
% % Z(j,i,5)=aa*t(5)*(3*(cx(5)*ones(NY,NX)-u(j,i)+cy(5)*ones(NY,NX)-u(j,i)) + 9*(uxuy5.*cx(5)+uxuy5.*cy(5))).*G(j,i);
% % Z(j,i,6)=aa*t(6)*(3*(cx(6)*ones(NY,NX)-u(j,i)+cy(6)*ones(NY,NX)-u(j,i)) + 9*(uxuy6.*cx(6)+uxuy6.*cy(6))).*G(j,i);
% % Z(j,i,7)=aa*t(7)*(3*(cx(7)*ones(NY,NX)-u(j,i)+cy(7)*ones(NY,NX)-u(j,i)) + 9*(uxuy7.*cx(7)+uxuy7.*cy(7))).*G(j,i);
% % Z(j,i,8)=aa*t(8)*(3*(cx(8)*ones(NY,NX)-u(j,i)+cy(8)*ones(NY,NX)-u(j,i)) + 9*(uxuy8.*cx(8)+uxuy8.*cy(8))).*G(j,i);
% Z(j,i,5)=aa*t(5)*(3*(2*ones(NY,NX)-u(j,i)) + 9*(2*uxuy5)).*G(j,i);
% Z(j,i,6)=aa*t(6)*(3*(0*ones(NY,NX)-u(j,i)) + 9*(0*uxuy6)).*G(j,i);
% Z(j,i,7)=aa*t(7)*(3*(-2*ones(NY,NX)-u(j,i)) + 9*(-2*uxuy7)).*G(j,i);
% Z(j,i,8)=aa*t(8)*(3*(0*ones(NY,NX)) + 9*(0*uxuy8)).*G(j,i);
% 
% Z(j,i,9)=aa*t(9)*(3*(-u(j,i))).*G(j,i);

Z(1,:,:)=0;Z(NY,:,:)=0; %No forcing on boundaries. Because: u=0 not explicity defined. REPORT!
% Z=0;
OMEGA=-omega*(f-feq) + Z;

fout=f + OMEGA;   %Relaxation towards local equilibrium

%STREAMING PHASE
G=rho.*g; %Forcing term
%Streaming of the particle distributions.
%Propagation of distributions to neighboring nodes. 
%ALTERNATIVE!
% NOT to be included when (2.30) is included(?!?!)
% fout(J,i,1) = fout(J,i,1) + 3*t(1)*G(J,i); 
% fout(J,i,3) = fout(J,i,3) - 3*t(3)*G(J,i);
% fout(J,i,5) = fout(J,i,5) + 3*t(5)*G(J,i);
% fout(J,i,6) = fout(J,i,6) - 3*t(6)*G(J,i);
% fout(J,i,7) = fout(J,i,7) - 3*t(7)*G(J,i);
% fout(J,i,8) = fout(J,i,8) + 3*t(8)*G(J,i);

fout(1,i,:)=f(1,i,:);   %The populations are simply bounced back at the UPPER
fout(NY,i,:)=f(NY,i,:); %...and at the LOWER wall. 

%################ "Old" streaming #############################
f(j,2:NX,1)            = fout(j,1:NX-1,1); %Propagation of c1
f(2:NY,i,2)            = fout(1:NY-1,i,2); %Propagation of c2
f(j,1:NX-1,3)          = fout(j,2:NX,3);   %Propagation of c3
f(1:NY-1,i,4)          = fout(2:NY,i,4);   %Propagation of c4
f(2:NY,2:NX,5)         = fout(1:NY-1,1:NX-1,5);%Propagation of c5
f(2:NY,1:NX-1,6)       = fout(1:NY-1,2:NX,6);%Propagation of c6
f(1:NY-1,1:NX-1,7)     = fout(2:NY,2:NX,7);%Propagation of c7
f(1:NY-1,2:NX,8)       = fout(2:NY,1:NX-1,8);%Propagation of c8
f(j,i,9)               = fout(j,i,9);%Propagation of c0. Necessary?


%APPLYING BOUNDARY CONDITIONS
%Periodic boundary conditions

f(j,1,1)        =   fout(j,NX,1);
f(j,NX,3)       =   fout(j,1,3);
f(2:NY,1,5)     =   fout(1:NY-1,NX,5);
f(2:NY,NX,6)    =   fout(1:NY-1,1,6);
f(1:NY-1,NX,7)  =   fout(2:NY,1,7);
f(1:NY-1,1,8)   =   fout(2:NY,NX,8);

%Bounceback boundary conditions
temp=f(1,i,2);
f(1,i,2)=f(1,i,4); %Upper boundary

temp=f(NY,i,4);
f(NY,i,4)=f(NY,i,2); %Lower boundary

temp=f(1,i,5);
f(1,i,5)=f(1,i,7); %Upper boundary

temp=f(NY,i,7);
f(NY,i,7)=f(NY,i,5); %Lower boundary

temp=f(1,i,6);
f(1,i,6)=f(1,i,8); %Upper boundary
% f(1,i,8)=temp;

temp=f(NY,i,8);
f(NY,i,8)=f(NY,i,6); %Lower boundary

n=n+1;
uprofile=jx(:,ceil(NY/2))./rho(:,ceil(NY/2))/u_lattice;
vprofile=jy(:,ceil(NY/2))./rho(:,ceil(NY/2))/u_lattice;
% hold off
% figure(1)
% plot(uprofile,'k+')

time(n)=delta_y*n;
xmomentum(n)=sum(jx(:,ceil(NY/2)));
errtemp=sqrt((uprofile(j)-uana(j)').^2+(vprofile(j)).^2);
err(n)=(1/NY)*sum(errtemp);
difference(n)=err(n-1)-err(n);
difference(n-1)=difference(n);


% pause(.1)
end
iterationtime=toc;
% N=1:n;
% hold off;
% figure(1)
% hold on; 
% plot(uprofile,'b-')
disp('#### RESULTS FROM POISEUILLE.M ####')
figure(1)
plot(uprofile,'k+')
hold on;
plot(uana,'k-')
xlabel('Grid node NY')
title(['Poiseuille flow with NY=',num2str(NY),' nodes'])
ylabel('u(NY)/ u_{max}')
legend('u(y)','u_{ana}')

figure(2)
plot(time,xmomentum,'k.')
xlabel('Time t')
ylabel('x-momentum j_x(t)')

% figure(2)
% loglog(NY,err(n),'k*')
% grid on
% title('2-norm of error for gravity driven Poiseuille flow')
% xlabel('NY')
% ylabel('Error \epsilon')
% axis([10^1 10^2 10^(-3) 10^0])
% hold on

disp(['Number of iterations required was ',num2str(n-1)])
disp(['Maximum velocity= ',num2str(max(uprofile))])
disp(['Error for NY= ',num2str(NY), 'nodes is ',num2str(err(n))])
disp(['iterationtime w/',num2str(n-1),' iterations was ',num2str(iterationtime)])

% hold off
% figure(4)
% semilogy(2:n,difference(2:n),'k.')
% title('Difference in error between iteration n and (n-1)')
% legend(['NY=',num2str(NY),' nodes'])
% xlabel('Iteration n')
% ylabel('Difference')







