function [u0_H0,normal_H0] = u0_comp(H0,r0,x0,y0,frekvens)
% Finds the velocity components, u0_x and u0_y of u0 at H0.

% u0: Inner wall velocity
% The inner wall velocity is constant when expressed in polar coordinates,
% that is, as a function of r (u0(r)=const). However, when expressed in
% cartesian coordinates, u0 has one x-component, u0_x, and one y-component,
% u0_y, where u0_x=u0_x(x,y) and u0_y=u0_y(x,y).

% u0 needs to be specified at all H0 and where r exists.

u0=frekvens*r0;
% For the velocity computation

for i=1:length(H0)

    nx_H0(i)=(H0(i,1)-x0)/r0;
    ny_H0(i)=(H0(i,2)-y0)/r0;
    
%     if nx_H0(i)~=0 && ny_H0(i)~=0
    
u0_x(i)=-(ny_H0(i)/abs(nx_H0(i))*u0)/(sqrt(1 + (ny_H0(i)/nx_H0(i))^2));
u0_y(i)= (nx_H0(i)/abs(ny_H0(i))*u0)/(sqrt(1 + (nx_H0(i)/ny_H0(i))^2));

    if nx_H0(i)==0
        u0_x(i)=-ny_H0(i).*u0;
        
    elseif ny_H0(i)==0
        u0_y(i)=nx_H0(i).*u0;
    end

end
u0_H0=[u0_x',u0_y'];
normal_H0=[nx_H0',ny_H0'];



