function [Sxx,Sxy,Syy,dudx,dudy,dvdx,dvdy] =rate_of_strain_bound(dudx,dudy,dvdx,dvdy,ux,uy,u0_r,r,boundary,p)
% rate_of_strain_bound computes S-tensor for the boundary nodes by means of 
% central finite differences. Go through each element in <r> or <u0_r>. If
% non-zero element, then take velocity from <u0_r>. The distance between 
% the points, |P2P1|, is computed from <p> and <r>.

for j=1:length(boundary)
% for j=1:10;
% j=2;
PP=boundary(j,:);
% j
a=PP(1);
b=PP(2);

     P_E=p(j,1:2); %Eastern neighbor
     P_N=p(j,3:4); %Northern neighbor
     P_W=p(j,5:6); %Western neighbor
     P_S=p(j,7:8); %Southern neighbor
     
     dudx(b,a)=0.5*ux(P_E(2),P_E(1))-0.5*ux(P_W(2),P_W(1));
     dudy(b,a)=0.5*ux(P_N(2),P_N(1))-0.5*ux(P_S(2),P_S(1));
     dvdx(b,a)=0.5*uy(P_E(2),P_E(1))-0.5*uy(P_W(2),P_W(1));
     dvdy(b,a)=0.5*uy(P_N(2),P_N(1))-0.5*uy(P_S(2),P_S(1));
     

for i=2:2:8 %Cardinal points: E,N,W,S
% i

     
if r(j,i)~=0 && i==2 %East

    dudx(b,a)=(u0_r(j,i-1) -  ux(P_W(2),P_W(1)))/abs(r(j,i-1)-P_W(1));
    dvdx(b,a)=(u0_r(j,i)   -  uy(P_W(2),P_W(1)))/abs(r(j,i-1)-P_W(1));

elseif r(j,i)~=0 && i==4 %North

    dudy(b,a)=(u0_r(j,i-1) -  ux(P_S(2),P_S(1)))/abs(r(j,i)-P_S(2));
    dvdy(b,a)=(u0_r(j,i)   -  uy(P_S(2),P_S(1)))/abs(r(j,i)-P_S(2));
    
elseif r(j,i)~=0 && i==6 %West
    
    dudx(b,a)=-(u0_r(j,i-1)-  ux(P_E(2),P_E(1)))/abs(r(j,i-1)-P_E(1));
    dvdx(b,a)=-(u0_r(j,i)  -  uy(P_E(2),P_E(1)))/abs(r(j,i-1)-P_E(1));
    
elseif r(j,i)~=0 && i==8 %South
    
    dudy(b,a)=-(u0_r(j,i-1) -  ux(P_N(2),P_N(1)))/abs(r(j,i)-P_N(2));
    dvdy(b,a)=-(u0_r(j,i)   -  uy(P_N(2),P_N(1)))/abs(r(j,i)-P_N(2));
        
end
end
end

Sxx=dudx;
Sxy=0.5*(dudy+dvdx);
Syy=dvdy;
